﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Text;
using TextBoxIDataErrorInfo.Rules;

namespace TextBoxIDataErrorInfo
{
    class ViewModeBase : INotifyPropertyChanged, IDataErrorInfo
    {
        private string error;
        public event PropertyChangedEventHandler PropertyChanged;
        protected void InvokePropertyChanged(string property)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged.Invoke(this, new PropertyChangedEventArgs(property));
            }
        }
        public string Error
        {
            get
            {
                return error;
            }
        }
        public string this[string propertyName]
        {
            get
            {
                var property = GetType().GetProperty(propertyName);
                if (property != null)
                {
                    var value = property.GetValue(this, null);
                    var attributes = property.GetCustomAttributes(false);
                    foreach (var attribute in attributes)
                    {
                        if (attribute is NumberRuleAttribute numberRule)
                        {
                            if (!numberRule.IsValid(value))
                            {
                                error = numberRule.ErrorMessage;
                                return error;
                            }
                        }
                        else if (attribute is StringRuleAttribute stringRule)
                        {
                            if (!stringRule.IsValid(value))
                            {
                                error = stringRule.ErrorMessage;
                                return error;
                            }
                        }
                    }
                }
                return null;
            }
        }

        public bool IsValidated()
        {

            // 通过反射获取所有带自定义特性的属性，判断属性值是否满足要求
            var properties = GetType().GetProperties(BindingFlags.Public| BindingFlags.Instance);
            foreach (var property in properties)
            {
                // 属性Error和Item来自接口IDataErrorInfo，无需进行验证
                if (property.Name == "Error" || property.Name == "Item")
                {
                    continue;
                }
                var value = property.GetValue(this, null);
                var attributes = property.GetCustomAttributes(false);
                foreach (var attribute in attributes)
                {
                    if (attribute is NumberRuleAttribute numberRule)
                    {
                        if (!numberRule.IsValid(value))
                        {
                            error = numberRule.ErrorMessage;
                            return false;
                        }
                    }
                    else if (attribute is StringRuleAttribute stringRule)
                    {
                        if (!stringRule.IsValid(value))
                        {
                            error = stringRule.ErrorMessage;
                            return false;
                        }
                    }
                }
            }
            return true;
        }

    }
}
